﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using MotorPlantDatabaseImplement.Models;


namespace MotorPlantDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
		public List<OrderViewModel> GetFullList()
		{
			using var context = new MotorPlantDatabase();
			return context.Orders
			.Select(x => AccessEngineStorage(x.GetViewModel))
		   .ToList();
		}
		public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
		{
			if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue)
			{
				return new();
			}
			using var context = new MotorPlantDatabase();
			if (model.Id.HasValue)
				return context.Orders.Where(x => x.Id == model.Id).Select(x => AccessEngineStorage(x.GetViewModel)).ToList();
			if (model.ClientId.HasValue)
				return context.Orders.Where(x => x.ClientId == model.ClientId).Select(x => AccessEngineStorage(x.GetViewModel)).ToList();
			return context.Orders.Where(x => x.DateCreate >= model.DateFrom).Where(x => x.DateCreate <= model.DateTo).
					Select(x => AccessEngineStorage(x.GetViewModel)).ToList();
		}
		public OrderViewModel? GetElement(OrderSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new MotorPlantDatabase();
			return AccessEngineStorage(context.Orders.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel);
		}
		public OrderViewModel? Insert(OrderBindingModel model)
		{
			var newOrder = Order.Create(model);
			if (newOrder == null)
			{
				return null;
			}
			using var context = new MotorPlantDatabase();
			context.Orders.Add(newOrder);
			context.SaveChanges();
			return AccessEngineStorage(newOrder.GetViewModel);
		}
		public OrderViewModel? Update(OrderBindingModel model)
		{
			using var context = new MotorPlantDatabase();
			var order = context.Orders.FirstOrDefault(x => x.Id ==
		   model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			context.SaveChanges();
			return AccessEngineStorage(order.GetViewModel);
		}
		public OrderViewModel? Delete(OrderBindingModel model)
		{
			using var context = new MotorPlantDatabase();
			var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				context.Orders.Remove(element);
				context.SaveChanges();
				return AccessEngineStorage(element.GetViewModel);
			}
			return null;
		}

		public static OrderViewModel AccessEngineStorage(OrderViewModel model)
		{
			if (model == null)
				return null;
			using var context = new MotorPlantDatabase();
			foreach (var Engine in context.Engines)
			{
				if (Engine.Id == model.EngineId)
				{
					model.EngineName = Engine.EngineName;
					break;
				}
			}
			foreach (var client in context.Clients)
			{
				if (client.Id == model.ClientId)
				{
					model.ClientFIO = client.ClientFIO;
					return model;
				}
			}
			return model;
		}
	}
}
