﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using MotorPlantFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorPlantFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
		private readonly DataFileSingleton source;
		public OrderStorage()
		{
			source = DataFileSingleton.GetInstance();
		}
		public List<OrderViewModel> GetFullList()
		{
			return source.Orders.Select(x => GetViewModel(x)).ToList();
		}
		public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
		{
			if (model.Id.HasValue)
				return source.Orders.Where(x => x.Id == model.Id)
					.Select(x => GetViewModel(x))
					.ToList();
			if (model.ClientId.HasValue)
				return source.Orders.Where(x => x.ClientId == model.ClientId).Select(x => GetViewModel(x)).ToList();
			return source.Orders.Where(x => x.DateCreate >= model.DateFrom).Where(x => x.DateCreate <= model.DateTo).
				   Select(x => GetViewModel(x)).ToList();
		}
		public OrderViewModel? GetElement(OrderSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			return GetViewModel(source.Orders.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id));
		}
		public OrderViewModel? Insert(OrderBindingModel model)
		{
			model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
			var newOrder = Order.Create(model);
			if (newOrder == null)
			{
				return null;
			}
			source.Orders.Add(newOrder);
			source.SaveOrders();
			return newOrder.GetViewModel;
		}
		public OrderViewModel? Update(OrderBindingModel model)
		{
			var component = source.Orders.FirstOrDefault(x => x.Id == model.Id);
			if (component == null)
			{
				return null;
			}
			component.Update(model);
			source.SaveOrders();
			return component.GetViewModel;
		}
		public OrderViewModel? Delete(OrderBindingModel model)
		{
			var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				source.Orders.Remove(element);
				source.SaveOrders();
				return element.GetViewModel;
			}
			return null;
		}
		private OrderViewModel GetViewModel(Order order)
		{
			var viewModel = order.GetViewModel;
			var engine = source.Engines.FirstOrDefault(x => x.Id == order.EngineId);
			var client = source.Clients.FirstOrDefault(x => x.Id == order.ClientId);
			viewModel.EngineName = engine?.EngineName;
			viewModel.ClientFIO = client?.ClientFIO;
			return viewModel;
		}
	}
}
