﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;

namespace MotorPlantRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class MainController : Controller
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _order;
        private readonly IEngineLogic _engine;
        public MainController(ILogger<MainController> logger, IOrderLogic order, IEngineLogic engine)
        {
            _logger = logger;
            _order = order;
            _engine = engine;
        }
        [HttpGet]
        public List<EngineViewModel>? GetEngineList()
        {
            try
            {
                return _engine.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка цветов");
                throw;
            }
        }
        [HttpGet]
        public EngineViewModel? GetEngine(int engineId)
        {
            try
            {
                return _engine.ReadElement(new EngineSearchModel
                {
                    Id = engineId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения движков по id = {Id}",engineId);
                throw;
            }
        }
        [HttpGet]
        public List<OrderViewModel>? GetOrders(int clientId)
        {
            try
            {
                return _order.ReadList(new OrderSearchModel
                {
                    ClientId = clientId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка заказов клиента id = {Id}", clientId);
            throw;
            }
        }
        [HttpPost]
        public void CreateOrder(OrderBindingModel model)
        {
            try
            {
                _order.CreateOrder(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                throw;
            }
        }
    }
}
